@@ -210,7 +210,9 @@ pango_attr_color_copy (const PangoAttribute *attr)
   const PangoAttrColor *color_attr = (PangoAttrColor *)attr;
   
   return pango_attr_color_new (attr->klass,
-			       color_attr->red, color_attr->green, color_attr->blue);
+			       color_attr->color.red,
+                               color_attr->color.green,
+                               color_attr->color.blue);
 }
 
 static void
@@ -225,10 +227,10 @@ pango_attr_color_equal (const PangoAttribute *attr1,
 {
   const PangoAttrColor *color_attr1 = (const PangoAttrColor *)attr1;
   const PangoAttrColor *color_attr2 = (const PangoAttrColor *)attr2;
-  
-  return (color_attr1->red == color_attr2->red &&
-	  color_attr1->blue == color_attr2->blue &&
-	  color_attr1->green == color_attr2->green);
+
+  return (color_attr1->color.red == color_attr2->color.red &&
+	  color_attr1->color.blue == color_attr2->color.blue &&
+	  color_attr1->color.green == color_attr2->color.green);
 }
 
 static PangoAttribute *
@@ -239,9 +241,9 @@ pango_attr_color_new (const PangoAttrClass *klass,
 {
   PangoAttrColor *result = g_new (PangoAttrColor, 1);
   result->attr.klass = klass;
-  result->red = red;
-  result->green = green;
-  result->blue = blue;
+  result->color.red = red;
+  result->color.green = green;
+  result->color.blue = blue;
 
   return (PangoAttribute *)result;
 }
@@ -681,6 +683,21 @@ pango_attr_shape_new (const PangoRectangle *ink_rect,
   return (PangoAttribute *)result;
 }
 
+GType
+pango_attr_list_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("PangoAttrList",
+					     NULL,
+					     pango_attr_list_copy,
+					     pango_attr_list_unref,
+					     FALSE);
+
+  return our_type;
+}
+
 /**
  * pango_attr_list_new:
  * 
@@ -801,6 +818,8 @@ pango_attr_list_insert_internal (PangoAttrList  *list,
 	   (!before && ((PangoAttribute *)list->attributes_tail->data)->start_index == start_index))
     {
       g_slist_append (list->attributes_tail, attr);
+      list->attributes_tail = list->attributes_tail->next;
+      g_assert (list->attributes_tail);
     }
   else
     {
@@ -990,6 +1009,9 @@ pango_attr_list_change (PangoAttrList  *list,
    * tmp_list points to prev->next.
    */
 
+  g_assert (prev->data == attr);
+  g_assert (prev->next == tmp_list);
+  
   /* We now have the range inserted into the list one way or the
    * other. Fix up the remainder
    */
@@ -1041,8 +1063,8 @@ pango_attr_list_change (PangoAttrList  *list,
 		  if (tmp_attr2->start_index >= tmp_attr->start_index)
 		    break;
 
+                  prev2 = tmp_list2;
 		  tmp_list2 = tmp_list2->next;
-		  prev2 = tmp_list2;
 		}
 
 	      /* Now remove and insert before tmp_list2. We'll
@@ -1474,3 +1496,40 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 	}
     }
 }
+
+GType
+pango_color_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("PangoColor",
+					     NULL,
+					     pango_color_copy,
+					     pango_color_free,
+					     FALSE);
+
+  return our_type;
+}
+
+PangoColor*
+pango_color_copy (const PangoColor *src)
+{
+  PangoColor *ret;
+
+  g_return_val_if_fail (src != NULL, NULL);
+  
+  ret = g_new (PangoColor, 1);
+
+  *ret = *src;
+
+  return ret;
+}
+
+void
+pango_color_free (PangoColor *color)
+{
+  g_return_if_fail (color != NULL);
+
+  g_free (color);
+}
