@@ -23,11 +23,31 @@
 #define __PANGO_ATTRIBUTES_H__
 
 #include <pango/pango-font.h>
+#include <glib-object.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
+/* PangoColor */
+
+typedef struct _PangoColor PangoColor;
+
+struct _PangoColor
+{
+  guint16 red;
+  guint16 green;
+  guint16 blue;
+};
+
+#define PANGO_TYPE_COLOR pango_color_get_type ()
+GType       pango_color_get_type (void) G_GNUC_CONST;
+
+PangoColor *pango_color_copy     (const PangoColor *src);
+void        pango_color_free     (PangoColor       *color);
+
+/* Attributes */
+
 typedef struct _PangoAttribute    PangoAttribute;
 typedef struct _PangoAttrClass    PangoAttrClass;
 				  
@@ -38,11 +58,13 @@ typedef struct _PangoAttrColor    PangoAttrColor;
 typedef struct _PangoAttrFontDesc PangoAttrFontDesc;
 typedef struct _PangoAttrShape    PangoAttrShape;
 
+#define PANGO_TYPE_ATTR_LIST pango_attr_list_get_type ()
 typedef struct _PangoAttrList     PangoAttrList;
 typedef struct _PangoAttrIterator PangoAttrIterator;
 
 typedef enum
 {
+  PANGO_ATTR_INVALID,           /* 0 is an invalid attribute type */
   PANGO_ATTR_LANG,		/* PangoAttrString */
   PANGO_ATTR_FAMILY,		/* PangoAttrString */
   PANGO_ATTR_STYLE,		/* PangoAttrInt */
@@ -103,9 +125,7 @@ struct _PangoAttrFloat
 struct _PangoAttrColor
 {
   PangoAttribute attr;
-  guint16 red;
-  guint16 green;
-  guint16 blue;
+  PangoColor color;
 };
 
 struct _PangoAttrShape
@@ -149,6 +169,7 @@ PangoAttribute *pango_attr_shape_new         (const PangoRectangle       *ink_re
 					      const PangoRectangle       *logical_rect);
 PangoAttribute *pango_attr_scale_new         (double                      scale_factor);
 
+GType              pango_attr_list_get_type      (void) G_GNUC_CONST;
 PangoAttrList *    pango_attr_list_new           (void);
 void               pango_attr_list_ref           (PangoAttrList  *list);
 void               pango_attr_list_unref         (PangoAttrList  *list);
