@@ -90,6 +90,8 @@ struct _PangoFontMetrics
 {
   int ascent;
   int descent;
+  int approximate_char_width;
+  int approximate_digit_width;
 };
 
 PangoFontDescription *pango_font_description_copy        (const PangoFontDescription  *desc);
@@ -101,6 +103,7 @@ void                  pango_font_descriptions_free       (PangoFontDescription
 
 PangoFontDescription *pango_font_description_from_string (const char                  *str);
 char *                pango_font_description_to_string   (const PangoFontDescription  *desc);
+char *                pango_font_description_to_filename (const PangoFontDescription  *desc);
 
 
 /* Logical fonts
