@@ -122,6 +122,16 @@ void pango_justify (PangoGlyphString *glyphs,
                     gint              new_line_width,
                     gint              min_kashida_width);
 
+
+/* Turn a string of characters into a string of glyphs
+ */
+void pango_shape (const gchar      *text,
+		  gint              length,
+		  PangoAnalysis    *analysis,
+		  PangoGlyphString *glyphs);
+
+GList *pango_reorder_items (GList *logical_items);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
