@@ -41,6 +41,8 @@
 
 #ifdef G_OS_WIN32
 
+#include <sys/types.h>
+
 #define STRICT
 #include <windows.h>
 
@@ -427,9 +429,9 @@ read_config_file (const char *filename, gboolean enoent_error)
 {
   FILE *file;
     
-  GString *line_buffer = g_string_new (NULL);
-  GString *tmp_buffer1 = g_string_new (NULL);
-  GString *tmp_buffer2 = g_string_new (NULL);
+  GString *line_buffer;
+  GString *tmp_buffer1;
+  GString *tmp_buffer2;
   char *errstring = NULL;
   const char *pos;
   char *section = NULL;
@@ -444,6 +446,10 @@ read_config_file (const char *filename, gboolean enoent_error)
       return;
     }
   
+  line_buffer = g_string_new (NULL);
+  tmp_buffer1 = g_string_new (NULL);
+  tmp_buffer2 = g_string_new (NULL);
+
   while (pango_read_line (file, line_buffer))
     {
       line++;
@@ -610,55 +616,33 @@ pango_config_key_get (const char *key)
   return g_strdup (g_hash_table_lookup (config_hash, key));
 }
 
-char *
+G_CONST_RETURN char *
 pango_get_sysconf_subdirectory (void)
 {
 #ifdef G_OS_WIN32
+  static gchar *result = NULL;
 
-  /* On Windows we don't hardcode any paths (SYSCONFDIR) in the DLL,
-   * but rely on an installation program to store the installation
-   * directory in the registry. If no installation program has been
-   * used, punt and assume the Pango directory is %WINDIR%\Pango.
-   */
-
-  static gboolean been_here = FALSE;
-  static gchar pango_sysconf_dir[200];
-  gchar win_dir[100];
-  HKEY reg_key = NULL;
-  DWORD type;
-  DWORD nbytes = sizeof (pango_sysconf_dir);
-
-  if (been_here)
-    return pango_sysconf_dir;
-
-  been_here = TRUE;
-
-  if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "Software\\GNU\\Pango", 0,
-		    KEY_QUERY_VALUE, &reg_key) != ERROR_SUCCESS
-      || RegQueryValueEx (reg_key, "InstallationDirectory", 0,
-			  &type, pango_sysconf_dir, &nbytes) != ERROR_SUCCESS
-      || type != REG_SZ)
-    {
-      /* Uh oh. Use %WinDir%\Pango */
-      GetWindowsDirectory (win_dir, sizeof (win_dir));
-      sprintf (pango_sysconf_dir, "%s\\pango", win_dir);
-    }
-
-  if (reg_key != NULL)
-    RegCloseKey (reg_key);
-
-  return pango_sysconf_dir;
+  if (result == NULL)
+    result = g_win32_get_package_installation_subdirectory
+      ("pango", g_strdup_printf ("pango-%s.dll", PANGO_VERSION), "etc\\pango");
 
+  return result;
 #else
   return SYSCONFDIR "/pango";
 #endif
 }
 
-char *
+G_CONST_RETURN char *
 pango_get_lib_subdirectory (void)
 {
 #ifdef G_OS_WIN32
-  return pango_get_sysconf_subdirectory ();
+  static gchar *result = NULL;
+
+  if (result == NULL)
+    result = g_win32_get_package_installation_subdirectory
+      ("pango", g_strdup_printf ("pango-%s.dll", PANGO_VERSION), "lib\\pango");
+
+  return result;
 #else
   return LIBDIR "/pango";
 #endif
