@@ -33,6 +33,7 @@
 #include "pango-utils.h"
 #include "pangoft2.h"
 #include "pangoft2-private.h"
+#include "modules.h"
 
 #define PANGO_TYPE_FT2_FONT              (pango_ft2_font_get_type ())
 #define PANGO_FT2_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT, PangoFT2Font))
@@ -101,12 +102,14 @@ pango_ft2_open_args_describe (PangoFT2OA *oa)
     return g_strdup_printf ("open_args at %p, face_index %ld", oa->open_args, oa->face_index);
 }
 
-static inline FT_Face
+FT_Face
 pango_ft2_get_face (PangoFont      *font,
 		    PangoFT2Subfont subfont_index)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
   PangoFT2FontCache *cache;
+  FT_Face face;
+  FT_Error error;
 
   if (subfont_index < 1 || subfont_index > ft2font->n_fonts)
     {
@@ -127,7 +130,19 @@ pango_ft2_get_face (PangoFont      *font,
 	g_warning ("Cannot load font for %s",
 		   pango_ft2_open_args_describe (ft2font->oa[subfont_index-1]));
     }
-  return ft2font->faces[subfont_index-1];
+
+  face = ft2font->faces[subfont_index-1];
+
+  if (ft2font->size != GPOINTER_TO_UINT (face->generic.data))
+    {
+      face->generic.data = GUINT_TO_POINTER (ft2font->size);
+
+      error = FT_Set_Char_Size (face, 0, PANGO_PIXELS_26_6 (ft2font->size), 72, 72);
+      if (error)
+	g_warning ("Error in FT_Set_Char_Size: %d", error);
+    }
+  
+  return face;
 }
 
 /**
@@ -141,7 +156,17 @@ PangoContext *
 pango_ft2_get_context (void)
 {
   PangoContext *result;
-
+  static gboolean registered_modules = FALSE;
+  int i;
+  
+  if (!registered_modules)
+    {
+      registered_modules = TRUE;
+      
+      for (i = 0; _pango_included_ft2_modules[i].list; i++)
+        pango_module_register (&_pango_included_ft2_modules[i]);
+    }
+  
   result = pango_context_new ();
   pango_context_add_font_map (result, pango_ft2_font_map_for_display ());
 
@@ -187,6 +212,7 @@ pango_ft2_font_init (PangoFT2Font *ft2font)
   ft2font->metrics_by_lang = NULL;
 
   ft2font->entry = NULL;
+  ft2font->glyph_info = g_hash_table_new (NULL, NULL);
 }
 
 static void
@@ -234,7 +260,7 @@ pango_ft2_load_font (PangoFontMap  *fontmap,
   g_return_val_if_fail (face_indices != NULL, NULL);
   g_return_val_if_fail (n_fonts > 0, NULL);
 
-  result = (PangoFT2Font *)g_type_create_instance (PANGO_TYPE_FT2_FONT);
+  result = (PangoFT2Font *)g_object_new (PANGO_TYPE_FT2_FONT, NULL);
   
   result->fontmap = fontmap;
   g_object_ref (G_OBJECT (result->fontmap));
@@ -278,7 +304,6 @@ pango_ft2_render (FT_Bitmap        *bitmap,
   int i;
   int x_position = 0;
   int ix, iy, ixoff, iyoff, y_start, y_limit, x_start, x_limit;
-  guint16 char_index;
   PangoFT2Subfont subfont_index;
   PangoGlyphInfo *gi;
   guchar *p, *q;
@@ -286,21 +311,20 @@ pango_ft2_render (FT_Bitmap        *bitmap,
   g_return_if_fail (bitmap != NULL);
   g_return_if_fail (glyphs != NULL);
 
-  PING (("bitmap: %dx%d@+%d+%d\n", bitmap->width, bitmap->rows, x, y));
+  PING (("bitmap: %dx%d@+%d+%d", bitmap->width, bitmap->rows, x, y));
 
   gi = glyphs->glyphs;
   for (i = 0; i < glyphs->num_glyphs; i++, gi++)
     {
       if (gi->glyph)
 	{
-	  char_index = PANGO_FT2_GLYPH_INDEX (gi->glyph);
+	  glyph_index = PANGO_FT2_GLYPH_INDEX (gi->glyph);
 	  subfont_index = PANGO_FT2_GLYPH_SUBFONT (gi->glyph);
 	  face = pango_ft2_get_face (font, subfont_index);
 
 	  if (face)
 	    {
 	      /* Draw glyph */
-	      glyph_index = FT_Get_Char_Index (face, char_index);
 	      /* FIXME hint or not? */ 
 	      FT_Load_Glyph (face, glyph_index, FT_LOAD_DEFAULT);
 	      FT_Render_Glyph (face->glyph, ft_render_mode_normal);
@@ -315,11 +339,11 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	      y_limit = MIN (face->glyph->bitmap.rows, face->glyph->bitmap_top - iyoff + bitmap->rows);
 
 
-	      PING (("glyph %d:%d: bitmap: %dx%d, left:%d top:%d\n",
+	      PING (("glyph %d:%d: bitmap: %dx%d, left:%d top:%d",
 		     i, glyph_index,
 		     face->glyph->bitmap.width, face->glyph->bitmap.rows,
 		     face->glyph->bitmap_left, face->glyph->bitmap_top));
-	      PING (("xstart:%d xlim:%d ystart:%d ylim:%d\n",
+	      PING (("xstart:%d xlim:%d ystart:%d ylim:%d",
 		     x_start, x_limit, y_start, y_limit));
 
 	      if (face->glyph->bitmap.pixel_mode == ft_pixel_mode_grays)
@@ -333,7 +357,7 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 		    q = face->glyph->bitmap.buffer + iy*face->glyph->bitmap.pitch;
 		    for (ix = x_start; ix < x_limit; ix++)
 		      {
-			*p = MIN (*p, 0xFF - *q);
+			*p = *q;
 			q++;
 			p++;
 		      }
@@ -374,24 +398,13 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 static FT_Glyph_Metrics *
 pango_ft2_get_per_char (PangoFont       *font,
 			PangoFT2Subfont  subfont_index,
-			guint16          char_index)
+			guint32          glyph_index)
 {
-  PangoFT2Font *ft2font = (PangoFT2Font *)font;
   FT_Face face;
-  FT_UInt glyph_index;
-  FT_Error error;
 
   if (!(face = pango_ft2_get_face (font, subfont_index)))
     return NULL;
 
-  glyph_index = FT_Get_Char_Index (face, char_index);
-  if (!glyph_index)
-    return NULL;
-
-  error = FT_Set_Pixel_Sizes (face, PANGO_PIXELS (ft2font->size), 0);
-  if (error)
-    g_warning ("Error in FT_Set_Pixel_Sizes: %d", error);
-
   FT_Load_Glyph (face, glyph_index, FT_LOAD_DEFAULT);
   return &face->glyph->metrics;
 }
@@ -402,50 +415,55 @@ pango_ft2_font_get_glyph_extents (PangoFont      *font,
 				  PangoRectangle *ink_rect,
 				  PangoRectangle *logical_rect)
 {
-  guint16 char_index = PANGO_FT2_GLYPH_INDEX (glyph);
-  PangoFT2Subfont subfont_index = PANGO_FT2_GLYPH_SUBFONT (glyph);
+  PangoFT2Font *ft2font = (PangoFT2Font *)font;
+  PangoFT2GlyphInfo *info;
+  PangoFT2Subfont subfont_index;
+  FT_UInt glyph_index;
   FT_Glyph_Metrics *gm;
 
-  if (glyph && (gm = pango_ft2_get_per_char (font, subfont_index, char_index)))
+  info = g_hash_table_lookup (ft2font->glyph_info, GUINT_TO_POINTER (glyph));
+
+  if (!info)
     {
-      if (ink_rect)
-	{
-	  ink_rect->x = PANGO_UNITS_26_6 (gm->horiBearingX);
-	  ink_rect->width = PANGO_UNITS_26_6 (gm->width);
-	  ink_rect->y = -PANGO_UNITS_26_6 (gm->horiBearingY);
-	  ink_rect->height = PANGO_UNITS_26_6 (gm->height);
-	}
-      if (logical_rect)
+      info = g_new (PangoFT2GlyphInfo, 1);
+      glyph_index = PANGO_FT2_GLYPH_INDEX (glyph);
+      subfont_index = PANGO_FT2_GLYPH_SUBFONT (glyph);
+      
+      if (glyph && (gm = pango_ft2_get_per_char (font, subfont_index, glyph_index)))
 	{
 	  FT_Face face = pango_ft2_get_face (font, subfont_index);
-
-	  logical_rect->x = 0;
-	  logical_rect->width = PANGO_UNITS_26_6 (gm->horiAdvance);
-	  logical_rect->y = -PANGO_UNITS_26_6 (face->size->metrics.ascender + 64);
+	  
+	  info->ink_rect.x = PANGO_UNITS_26_6 (gm->horiBearingX);
+	  info->ink_rect.width = PANGO_UNITS_26_6 (gm->width);
+	  info->ink_rect.y = -PANGO_UNITS_26_6 (gm->horiBearingY);
+	  info->ink_rect.height = PANGO_UNITS_26_6 (gm->height);
+	      
+	  info->logical_rect.x = 0;
+	  info->logical_rect.width = PANGO_UNITS_26_6 (gm->horiAdvance);
+	  info->logical_rect.y = -PANGO_UNITS_26_6 (face->size->metrics.ascender + 64);
 	  /* Some fonts report negative descender, some positive ! (?) */
-	  logical_rect->height = PANGO_UNITS_26_6 (face->size->metrics.ascender + ABS (face->size->metrics.descender) + 128);
+	  info->logical_rect.height = PANGO_UNITS_26_6 (face->size->metrics.ascender + ABS (face->size->metrics.descender) + 128);
 	}
-      PING (("glyph:%d logical: %dx%d@%d+%d\n",
-	     char_index, logical_rect->width, logical_rect->height,
-	     logical_rect->x, logical_rect->y));
-    }
-  else
-    {
-      if (ink_rect)
+      else
 	{
-	  ink_rect->x = 0;
-	  ink_rect->width = 0;
-	  ink_rect->y = 0;
-	  ink_rect->height = 0;
-	}
-      if (logical_rect)
-	{
-	  logical_rect->x = 0;
-	  logical_rect->width = 0;
-	  logical_rect->y = 0;
-	  logical_rect->height = 0;
+	  info->ink_rect.x = 0;
+	  info->ink_rect.width = 0;
+	  info->ink_rect.y = 0;
+	  info->ink_rect.height = 0;
+
+	  info->logical_rect.x = 0;
+	  info->logical_rect.width = 0;
+	  info->logical_rect.y = 0;
+	  info->logical_rect.height = 0;
 	}
+
+      g_hash_table_insert (ft2font->glyph_info, GUINT_TO_POINTER(glyph), info);
     }
+  
+  if (ink_rect)
+    *ink_rect = info->ink_rect;
+  if (logical_rect)
+    *logical_rect = info->logical_rect;
 }
 
 int
@@ -454,8 +472,6 @@ pango_ft2_font_get_kerning (PangoFont *font,
 			    PangoGlyph right)
 {
   PangoFT2Subfont subfont_index;
-  guint16 left_char_index;
-  guint16 right_char_index;
   FT_Face face;
   FT_UInt left_glyph_index, right_glyph_index;
   FT_Error error;
@@ -472,12 +488,10 @@ pango_ft2_font_get_kerning (PangoFont *font,
   if (!FT_HAS_KERNING (face))
     return 0;
 
-  left_char_index = PANGO_FT2_GLYPH_INDEX (left);
-  right_char_index = PANGO_FT2_GLYPH_INDEX (right);
+  left_glyph_index = PANGO_FT2_GLYPH_INDEX (left);
+  right_glyph_index = PANGO_FT2_GLYPH_INDEX (right);
 
-  left_glyph_index = FT_Get_Char_Index (face, left_char_index);
-  right_glyph_index = FT_Get_Char_Index (face, right_char_index);
-  if (!left_glyph_index || !right_char_index)
+  if (!left_glyph_index || !right_glyph_index)
     return 0;
 
   error = FT_Get_Kerning (face, left_glyph_index, right_glyph_index,
@@ -510,14 +524,14 @@ get_font_metrics_from_subfonts (PangoFont        *font,
 
       if (first)
 	{
-	  metrics->ascent = PANGO_UNITS_26_6 (face->ascender);
-	  metrics->descent = PANGO_UNITS_26_6 (face->descender);
+	  metrics->ascent = PANGO_UNITS_26_6 (face->size->metrics.ascender);
+	  metrics->descent = PANGO_UNITS_26_6 (-face->size->metrics.descender);
 	  first = FALSE;
 	}
       else
 	{
-	  metrics->ascent = MAX (PANGO_UNITS_26_6 (face->ascender), metrics->ascent);
-	  metrics->descent = MAX (PANGO_UNITS_26_6 (face->descender), metrics->descent);
+	  metrics->ascent = MAX (PANGO_UNITS_26_6 (face->size->metrics.ascender), metrics->ascent);
+	  metrics->descent = MAX (PANGO_UNITS_26_6 (-face->size->metrics.descender), metrics->descent);
 	}
 
       tmp_list = tmp_list->next;
@@ -547,9 +561,7 @@ get_font_metrics_from_string (PangoFont        *font,
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   GSList *subfonts = NULL;
   
-  n_chars = g_utf8_strlen (str, -1);
-
-  text_ucs4 = g_utf8_to_ucs4 (str, strlen (str));
+  text_ucs4 = g_utf8_to_ucs4_fast (str, -1, &n_chars);
   if (!text_ucs4)
     return;
 
@@ -730,33 +742,6 @@ pango_ft2_n_subfonts (PangoFont *font)
   return ft2font->n_fonts;
 }
 
-/**
- * pango_ft2_has_glyph:
- * @font: a #PangoFont which must be from the FreeType2 backend.
- * @glyph: the index of a glyph in the font. (Formed
- *         using the PANGO_FT2_MAKE_GLYPH macro)
- * 
- * Check if the given glyph is present in a FT2 font.
- * 
- * Return value: %TRUE if the glyph is present.
- **/
-gboolean
-pango_ft2_has_glyph (PangoFont  *font,
-		     PangoGlyph  glyph)
-{
-  guint16 char_index = PANGO_FT2_GLYPH_INDEX (glyph);
-  PangoFT2Subfont subfont_index = PANGO_FT2_GLYPH_SUBFONT (glyph);
-  FT_Face face = pango_ft2_get_face (font, subfont_index);
-
-  if (!face)
-    return FALSE;
-  
-  if (FT_Get_Char_Index (face, char_index) == 0)
-    return FALSE;
-  else
-    return TRUE;
-}
-
 PangoCoverage *
 pango_ft2_get_coverage (PangoFont  *font,
 			const char *lang)
@@ -768,32 +753,17 @@ pango_ft2_get_coverage (PangoFont  *font,
   FT_Face face;
   gunichar wc;
   int i;
-  GTimeVal tv0, tv1;
 
   for (i = 1; i <= ft2font->n_fonts; i++)
     {
-      g_get_current_time (&tv0);
       tmp = pango_coverage_new ();
       face = pango_ft2_get_face (font, i);
-      g_get_current_time (&tv1);
-      if (tv1.tv_usec < tv0.tv_usec)
-	tv1.tv_sec--, tv1.tv_usec += 1000000L;
-      g_print ("after pango_ft2_get_face: %ld.%06ld s\n", tv1.tv_sec - tv0.tv_sec, tv1.tv_usec - tv0.tv_usec);
       for (wc = 0; wc < 65536; wc++)
 	{
 	  glyph = PANGO_FT2_MAKE_GLYPH (i, wc);
 	  if (FT_Get_Char_Index (face, wc))
 	    pango_coverage_set (tmp, wc, PANGO_COVERAGE_EXACT);
-	  if ((wc%3000)==1) {
-      g_get_current_time (&tv1);
-      if (tv1.tv_usec < tv0.tv_usec)
-	tv1.tv_sec--, tv1.tv_usec += 1000000L;
-      g_print ("after %d loops: %ld.%06ld s\n", wc, tv1.tv_sec - tv0.tv_sec, tv1.tv_usec - tv0.tv_usec);}
 	}
-      g_get_current_time (&tv1);
-      if (tv1.tv_usec < tv0.tv_usec)
-	tv1.tv_sec--, tv1.tv_usec += 1000000L;
-      g_print ("after loop for %s: %ld.%06ld s\n", face->family_name, tv1.tv_sec - tv0.tv_sec, tv1.tv_usec - tv0.tv_usec);
       pango_coverage_max (result, tmp);
       pango_coverage_unref (tmp);
     }
@@ -848,6 +818,13 @@ pango_ft2_font_shutdown (GObject *object)
 }
 
 
+static gboolean
+pango_ft2_free_glyph_info_callback (gpointer key, gpointer value, gpointer data)
+{
+  g_free (value);
+  return TRUE;
+}
+
 static void
 pango_ft2_font_finalize (GObject *object)
 {
@@ -855,7 +832,7 @@ pango_ft2_font_finalize (GObject *object)
   PangoFT2FontCache *cache = pango_ft2_font_map_get_font_cache (ft2font->fontmap);
   int i;
 
-  PING (("\n"));
+  PING ((" "));
 
   for (i = 0; i < ft2font->n_fonts; i++)
     {
@@ -874,14 +851,24 @@ pango_ft2_font_finalize (GObject *object)
 
   g_object_unref (G_OBJECT (ft2font->fontmap));
 
+  g_hash_table_foreach_remove (ft2font->glyph_info, pango_ft2_free_glyph_info_callback, NULL);
+  g_hash_table_destroy (ft2font->glyph_info);
+  
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static PangoFontDescription *
 pango_ft2_font_describe (PangoFont *font)
 {
-  /* FIXME: implement */
-  return NULL;
+  PangoFT2Font         *ft2font;
+  PangoFontDescription *desc;
+
+  ft2font = PANGO_FT2_FONT (font);
+
+  desc = pango_font_description_copy (&ft2font->entry->description);
+  desc->size = ft2font->size;
+
+  return desc;
 }
 
 PangoMap *
@@ -992,27 +979,22 @@ pango_ft2_render_layout_line (FT_Bitmap       *bitmap,
 	    (y + 4) * bitmap->pitch +
 	    x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
 
-	  /* Don't drawn the underline through descenders */
 	  for (ix = 0; ix < x_limit; ix++)
-	    if (*p == 0xFF &&
-		(ix == x_limit - 1 || p[1] == 0xFF))
-	      *p++ = 0;
+	      *p++ = 0xff;
 	  /* Fall through */
 	case PANGO_UNDERLINE_SINGLE:
 	  p = bitmap->buffer +
 	    (y + 2) * bitmap->pitch +
 	    x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
 	  for (ix = 0; ix < x_limit; ix++)
-	    if (*p == 0xFF &&
-		(ix == x_limit - 1 || p[1] == 0xFF))
-	      *p++ = 0;
+	      *p++ = 0xff;
 	  break;
 	case PANGO_UNDERLINE_LOW:
 	  p = bitmap->buffer +
 	    (y + PANGO_PIXELS (ink_rect.y + ink_rect.height)) * bitmap->pitch +
 	    x + PANGO_PIXELS (x_off + ink_rect.x) - 1;
 	  for (ix = 0; ix < PANGO_PIXELS (ink_rect.width); ix++)
-	    *p++ = 0;
+	    *p++ = 0xff;
 	  break;
 	}
 
@@ -1051,7 +1033,7 @@ pango_ft2_render_layout (FT_Bitmap   *bitmap,
   width = pango_layout_get_width (layout);
   align = pango_layout_get_alignment (layout);
 
-  PING (("x:%d y:%d indent:%d width:%d\n", x, y, indent, width));
+  PING (("x:%d y:%d indent:%d width:%d", x, y, indent, width));
 
   if (width == -1 && align != PANGO_ALIGN_LEFT)
     {
@@ -1097,7 +1079,7 @@ pango_ft2_render_layout (FT_Bitmap   *bitmap,
 	    }
 	}
 
-      PING (("x_offset:%d y_offset:%d logical_rect.y:%d logical_rect.height:%d\n", x_offset, y_offset, logical_rect.y, logical_rect.height));
+      PING (("x_offset:%d y_offset:%d logical_rect.y:%d logical_rect.height:%d", x_offset, y_offset, logical_rect.y, logical_rect.height));
 
       pango_ft2_render_layout_line (bitmap, line,
 				    x + PANGO_PIXELS (x_offset),
@@ -1177,14 +1159,13 @@ ft_error_compare (const void *pkey,
 const char *
 pango_ft2_ft_strerror (FT_Error error)
 {
-#undef FTERRORS_H
+#undef __FTERRORS_H__
 #define FT_ERRORDEF( e, v, s )  { e, s },
 #define FT_ERROR_START_LIST  {
-#define FT_ERROR_END_LIST    { 0, 0 } }
+#define FT_ERROR_END_LIST    { 0, 0 } };
 
   const ft_error_description ft_errors[] =
-#include <freetype/fterrors.h>
-  ;
+#include FT_ERRORS_H
 
 #undef FT_ERRORDEF
 #undef FT_ERROR_START_LIST
