@@ -26,11 +26,10 @@
 #include <stdlib.h>
 #include <glib.h>
 
-#include <fribidi/fribidi.h>
-
 #include "pango-utils.h"
 #include "pangowin32.h"
 #include "pangowin32-private.h"
+#include "modules.h"
 
 #define PANGO_TYPE_WIN32_FONT            (pango_win32_font_get_type ())
 #define PANGO_WIN32_FONT(object)         (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_WIN32_FONT, PangoWin32Font))
@@ -173,6 +172,16 @@ PangoContext *
 pango_win32_get_context (void)
 {
   PangoContext *result;
+  static gboolean registered_modules = FALSE;
+  int i;
+
+  if (!registered_modules)
+    {
+      registered_modules = TRUE;
+      
+      for (i = 0; _pango_included_win32_modules[i].list; i++)
+        pango_module_register (&_pango_included_win32_modules[i]);
+    }
 
   result = pango_context_new ();
   pango_context_add_font_map (result, pango_win32_font_map_for_display ());
@@ -557,8 +566,7 @@ pango_win32_unicode_classify (wchar_t wc)
       else
 	return -1;
     }
-  /* NOTREACHED */
-  return 0;
+  return PANGO_WIN32_U_LAST_PLUS_ONE;
 }
 
 static void
@@ -680,7 +688,7 @@ get_font_metrics_from_string (PangoFont        *font,
   
   n_chars = g_utf8_strlen (str, -1);
 
-  text_ucs4 = g_utf8_to_ucs4 (str, strlen (str));
+  text_ucs4 = g_utf8_to_ucs4 (str, strlen (str), NULL, NULL, NULL);
   if (!text_ucs4)
     return;
 
@@ -989,13 +997,18 @@ subfont_has_glyph (PangoFont             *font,
 {
   TEXTMETRIC tm;
   PangoWin32Font *win32font = (PangoWin32Font *) font;
+  PangoWin32UnicodeSubrange subrange;
   wchar_t default_wc;
 #ifdef HEAVY_DEBUGGING
   static int dispx = 5, dispy = 0;
 #endif
 
+  subrange = pango_win32_unicode_classify (wc);
+  if (PANGO_WIN32_U_LAST_PLUS_ONE == subrange)
+    return FALSE;
+
   if (!pango_win32_logfont_has_subrange (win32font->fontmap, &info->logfont,
-					 pango_win32_unicode_classify (wc)))
+					 subrange))
     return FALSE;
 
   if (info->buf_hbm == NULL)
@@ -1044,6 +1057,19 @@ subfont_has_glyph (PangoFont             *font,
       SelectObject (info->buf_hdc, info->buf_hbm);
     }
 
+
+#if 0 /* This doesn't work either */
+  /* GetTextMetricsW unsupported on Win9x, questionable results
+   * on NT 4.0
+   */
+  if (GetTextMetricsW (info->buf_hdc, &tmw) 
+      && ((tmw.tmLastChar < wc) || (wc < tmw.tmFirstChar)))
+  {
+    //g_print("ShortCut %d %d %d\n", wc, tmw.tmFirstChar, tmw.tmLastChar);
+    return FALSE;
+  }
+#endif
+
   /* Draw character into our bitmap; compare with the bitmap for
    * the default character. If they are identical, this character
    * does not exist in the font.
@@ -1195,8 +1221,10 @@ pango_win32_font_finalize (GObject *object)
 static PangoFontDescription *
 pango_win32_font_describe (PangoFont *font)
 {
-  /* FIXME: implement */
-  return NULL;
+  PangoFontDescription *result;
+  PangoWin32Font *win32font = PANGO_WIN32_FONT (font);
+
+  return pango_font_description_copy (&win32font->entry->description);
 }
 
 PangoMap *
@@ -1328,9 +1356,9 @@ pango_win32_render_layout_line (HDC               hdc,
 	{
 	  HBRUSH oldbrush;
 
-	  brush = CreateSolidBrush (RGB ((bg_color.red + 128) >> 8,
-					 (bg_color.green + 128) >> 8,
-					 (bg_color.blue + 128) >> 8));
+	  brush = CreateSolidBrush (RGB ((bg_color.color.red + 128) >> 8,
+					 (bg_color.color.green + 128) >> 8,
+					 (bg_color.color.blue + 128) >> 8));
 	  oldbrush = SelectObject (hdc, brush);
 	  Rectangle (hdc, x + (x_off + logical_rect.x) / PANGO_SCALE,
 			  y + overall_rect.y / PANGO_SCALE,
@@ -1342,9 +1370,9 @@ pango_win32_render_layout_line (HDC               hdc,
 
       if (fg_set)
 	{
-	  brush = CreateSolidBrush (RGB ((fg_color.red + 128) >> 8,
-					 (fg_color.green + 128) >> 8,
-					 (fg_color.blue + 128) >> 8));
+	  brush = CreateSolidBrush (RGB ((fg_color.color.red + 128) >> 8,
+					 (fg_color.color.green + 128) >> 8,
+					 (fg_color.color.blue + 128) >> 8));
 	  oldfg = SelectObject (hdc, brush);
 	}
 
