@@ -24,7 +24,7 @@
 #define __PANGOX_H__
 
 #include <glib.h>
-#include <pango/pango.h>
+#include <pango/pango-layout.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -34,7 +34,7 @@ extern "C" {
 
 #define PANGO_RENDER_TYPE_X "PangoRenderX"
 
-typedef GC (*PangoGetGCFunc) (PangoContext *context, PangoAttrColor *color, GC base_gc);
+typedef GC (*PangoGetGCFunc) (PangoContext *context, PangoColor *color, GC base_gc);
 typedef void (*PangoFreeGCFunc) (PangoContext *context, GC gc);
 
 /* Calls for applications
@@ -92,7 +92,7 @@ PangoXFontCache *pango_x_font_cache_new     (Display         *display);
 void             pango_x_font_cache_free    (PangoXFontCache *cache);
 
 XFontStruct * pango_x_font_cache_load      (PangoXFontCache *cache,
-					    char            *xlfd);
+					    const char      *xlfd);
 void          pango_x_font_cache_unload    (PangoXFontCache *cache,
 					    XFontStruct     *fs);
 
